#!/bin/bash

SMIL2WavHeaderSize=40
DVFrameSize=144000

# ----------------------------------------------------------
# COMMUNICATION ROUTINES FOR USE WITH KOMMANDER GUI
# ----------------------------------------------------------

# Set the currently executing task to $1.

UpdateTask ()
{
 if [ -n "$DialogDCOPTaskCommand" ]; then
  $DialogDCOPTaskCommand "$1"
 fi
}

# Update progress to $1 %.

UpdateProgress ()
{
 if [ -n "$DialogDCOPProgressCommand" ]; then
   $DialogDCOPProgressCommand $1
 fi
}


# ----------------------------------------------------------
# ROUTINES TO COMMUNICATE ERRORS/WARNINGS TO THE USER
# ----------------------------------------------------------


# Present an error message to the user
# Argument $1 is error message.

PrintError ()
{
 X=`type -p kdialog`

 if test $X && test -x $X && [ -n "$DISPLAY" ]; then
  kdialog --error "$1"
 else
  echo
  echo "---- RECEIVED ERROR MESSAGE ----"
  echo
  echo -e "$1"
  echo
  echo "--------------------------------"
 fi
 exit -1
}

# Present a warning message to the user
# Argument $1 is warning message.

PrintWarning ()
{
 X=`type -p kdialog`

 if test $X && test -x $X && [ -n "$DISPLAY" ]; then
  kdialog --warningcontinuecancel "$1"
  if [ ! "$?" -eq 0 ]; then
  echo
  echo "USER ABORTED PROGRAM"
  echo
  exit -1
  fi
  
 else
  echo
  echo "---- WARNING ----"
  echo
  echo -e "$1"
  echo
  echo "-----------------"
 fi
}

# ----------------------------------------------------------
# CHECKS TO SEE IF WE HAVE ALL THE REQUIRED TOOLS AVAILABLE.
# ----------------------------------------------------------

Requirements=( convert composite mogrify identify sox mplayer smil2raw raw2image ppm2raw ppmfilter )

PrettyRequirements=( "ImageMagick >= 6" "Sox" "MPlayer >= 1.0pre4" "SMIL Utils" )

# Check if tool $1 is available.

ToolCheck ()
{
 echo -ne "Checking for \"$1\" ...\t"
	X=`type -p $1`
	if test $X && test -x $X;
		then echo " found." ; return 0
		else echo " MISSING" ; return 1
	fi
}

# Print an error when a required utility is missing

Missing ()
{
 PrintError "You don't have the utility \"$1\" installed! \n\nTo use this progam, you need to have at least these packages installed:\n\n`
 
 Requirement=0
 while [ -n "${PrettyRequirements[$Requirement]}" ]
 do
  echo -n "* ${PrettyRequirements[$Requirement]}\n"
  Requirement=$[$Requirement + 1]
 done
 
 
 `" 
 exit -1
}

RunChecks ()
{
 Requirement=0
 while [ -n "${Requirements[$Requirement]}" ]
 do
   ToolCheck "${Requirements[$Requirement]}" || Missing "${Requirements[$Requirement]}"
   Requirement=$[$Requirement + 1]
 done
 echo
}


# ----------------------------------------------------------
# DISK SPACE CHECKING ROUTINES
# ----------------------------------------------------------

# Warn if disk space in directory $1 lower than $2 MB

DiskSpaceWarning ()
{
  FreeSpace=`df  -m "$1" -P | awk '{print $4}' | tail -n 1`

  if [ $FreeSpace -lt $2 ]; then
  PrintWarning "You are low on disk space in the following directory:\n\n$1\n\nTo be completely safe, you are advised to free more disk space before continuing."
  fi
}

# Abort if disk space ran out in directory $1

DiskSpaceError ()
{
  FreeSpace=`df  -m "$1" -P | awk '{print $4}' | tail -n 1`
  
  if [ $FreeSpace -le 1 ]; then
  PrintError "This program ran out of disk space in the following directory:\n\n$1\n\nPlease free some more disk space and try again."
  fi
}


# ----------------------------------------------------------
#   FRAME GENERATION ROUTINES
# ----------------------------------------------------------



# Scales/Crops image $1 to fit into the DVD picture size

CropAndFit ()
{
     YRes=`identify "$1" | awk -Fx '{print $NF}' | awk -F+ '{print $1}' | awk '{print $1}'`
     echo "YRES: $YRes"
     XRes=`identify "$1" | awk -Fx '{print $(NF-1)}' | awk '{print $NF}' | awk '{print $1}'`
     echo "XRES: $XRes"
     
     UpdateProgress `echo "($Slide * 100) / $NumSlides" | bc`
     
     if [ $XRes -gt $YRes ]; then
        echo "Scaling..."
        convert "$1" -resize ${Width} "$1"
        YRes=`identify "$1" | awk -Fx '{print $NF}' | awk -F+ '{print $1}' | awk '{print $1}'`
        if [ $YRes -gt 576 ]; then
          echo "Cropping..."
          convert "$1" -crop ${Width}x${Height}+0+$[$[$YRes - 576] / 2] "$1"
        fi
        if [ $YRes -lt 576 ]; then
          echo "Expanding..."
          convert \( -size ${Width}x${Height} xc:'#000000FF' "$1" -geometry +0+$[$[576 - $YRes] / 2] -compose over -composite \) "$1"  
        fi
     else
        echo "Scaling..."
        convert "$1" -resize x${Height} "$1"
        XRes=`identify "$1" | awk -Fx '{print $(NF-1)}' | awk '{print $NF}' | awk '{print $1}'`
        if [ $XRes -gt $Width ]; then
          echo "Cropping..."
          convert "$1" -crop ${Width}x${Height}+$[$[$XRes - $Width] / 2]+0 "$1"
        fi
        if [ $XRes -lt $Width ]; then
          echo "Expanding..."
          convert \( -size ${Width}x${Height} xc:'#000000FF' "$1" -geometry +$[$[$Width - $XRes] / 2]+0 -compose over -composite \) "$1"  
        fi
     fi
 
}

# Scale/Crop/Expand slides to fit DVD picture size

ProcessSlides ()
{
 rm -f -R "$WorkingDir/Slides"
 mkdir -p "$WorkingDir/Slides"
 
 UpdateTask "Preprocessing Slides..."
 
 for ((Slide=1; Slide <= NumSlides ; Slide++))
   do
    cp "${SlidePath[$Slide]}" "$WorkingDir/Slides/Slide$Slide.png"
    CropAndFit "$WorkingDir/Slides/Slide$Slide.png"
   
    if [ ${SlideFlickerReduction[$Slide]} -gt 0 ]; then
    
      ReducedHeight=$(( $Height/2 + ( ( $Height/2 ) * (99 - ${SlideFlickerReduction[$Slide]}) ) / 99 ))
    
      echo "Reduced Height is $ReducedHeight"
      
      # Reduce vertical resolution to avoid flicker
      
      convert "$WorkingDir/Slides/Slide$Slide.png" -filter Sinc -resize ${Width}x${ReducedHeight}! -filter Sinc -resize ${Width}x${Height}! "$WorkingDir/Slides/Slide$Slide.png"
    
    fi
   
   done
 
 # Also process background slide
 if [ "$BackgroundType" == "Picture" ]; then
 rm -f "$WorkingDir/background.png"
 convert "$BackgroundImage" "$WorkingDir/background.png"
 CropAndFit "$WorkingDir/background.png"
 fi
 
 UpdateProgress 0
 UpdateTask "Done."
}

CreateAudioTrack ()
{  

 # We add sound from an external file
 if [ $SoundType == "SoundFile" ]; then
   
   UpdateTask "Decoding Soundtrack File..."
   echo "Decoding Soundtrack..."
   
   cd "$OutputDir"
   mplayer "$SoundtrackFile" -ao pcm -vc dummy -vo null < /dev/null > /dev/null 2>&1
   DiskSpaceError "$OutputDir"
   
   DetectedSampleRate=`sox -V -t .wav "$OutputDir/audiodump.wav" -e 2>&1 | grep -E -o [0-9]{1,\}\ samp/sec | awk '{print $1}'`
   
   # Fadeout soundtrack at the end
   
   Length=`echo "scale=4; tmp = $GlobalFrameCounter / 25; if (tmp <= 1) tmp = 1; tmp" | bc`
   FadeoutLength=`echo "scale=4; tmp = $Length * 0.05; if (tmp <= 1) tmp=1; tmp" | bc`
   
   UpdateTask "Processing Soundtrack File..."
   
   if [ "$DetectedSampleRate" == 48000 ]; then
   
     echo "Soundtrack samplerate is 48 KHz, no resampling needed."
   
     sox "$OutputDir/audiodump.wav" -t .wav -r 48000 /dev/stdout fade t 0 $Length  $FadeoutLength trim 0 $Length > "$WorkingDir/Soundtrack.wav"
   
   elif [ -z "$DetectedSampleRate" ]; then
   
     PrintError "Something is wrong with the selected soundtrack file:\n\n$SoundtrackFile\n\nCould not detect the correct sample rate."
   
   else
   
     echo "Soundtrack samplerate is not 48 KHz, resampling..."
     
     sox "$OutputDir/audiodump.wav" -t .wav -r 48000 /dev/stdout resample fade t 0 $Length  $FadeoutLength trim 0 $Length > "$WorkingDir/Soundtrack.wav"
   
   fi
   
   DiskSpaceError "$OutputDir"
   rm -f "$OutputDir/audiodump.wav"

 fi
 
 # We add sound from background movie
 if [ $SoundType == "MovieSound" ] && [ ! -z "$BackgroundMovie" ]; then
 
   echo "Copying soundtrack from background video..."
   UpdateTask "Extracting sound from background video..."
   
   # Extract the header from the soundtrack of the DV video
   smil2wav "$BackgroundMovie" 2>/dev/null | dd bs=$SMIL2WavHeaderSize count=1 of="$WorkingDir/header" 
  
   # Examine header, find sample rate
   SampleRate=`cat "$WorkingDir/header" | sox -V -t .wav /dev/stdin -e 2>&1 | grep -E -o [0-9]{1,\}\ samp/sec | awk '{print $1}'`
  
   if [ -z "$SampleRate" ]; then
     PrintError "Something is wrong with the soundtrack of video file $1.\n\nFailed to detect the correct sample rate."
   else
     echo "Detected sample rate of $SampleRate"
   fi

   if [ ! $SampleRate == 48000 ]; then
    
     smil2wav "$BackgroundMovie" 2>/dev/null | dd bs=$SMIL2WavHeaderSize skip=1 | sox -r $SampleRate -s -c  2 -w -t .raw /dev/stdin -r 48000 "$WorkingDir/Soundtrack.wav" resample
  
   else
  
     smil2wav "$BackgroundMovie" 2>/dev/null | dd bs=$SMIL2WavHeaderSize skip=1 | sox -r $SampleRate -s -c  2 -w -t .raw /dev/stdin "$WorkingDir/Soundtrack.wav"
  
   fi


   DiskSpaceError "$OutputDir"

 fi   
 UpdateTask "Done."
}

EncodeToDV ()
{
  rm -f "$WorkingDir/Slideshow001.avi"

  DiskSpaceWarning "$OutputDir" `echo "$TotalFrames * 0.21 + 1" | bc | cut -d . -f 1`

  UpdateTask "Encoding to Sony DV Video..."


  if [ $WideScreen == "true" ]; then
    WideScreenSwitch="-w"
  fi

  # Overlay frames on top of the DV background

  if [ "$BackgroundType" == "Movie" ]; then

    cd "$WorkingDir"

    if [ -e "$WorkingDir/Soundtrack.wav" ]; then

      smil2raw $WideScreenSwitch "$BackgroundMovie" | raw2image -ppm | ppmfilter --stack-image Frames/* period=1 --overlay position=0.0,0.0:100.0x100.0 | ppm2raw -A "$WorkingDir/Soundtrack.wav" > "$OutputDir/Slideshow.dv"

    else

      smil2raw $WideScreenSwitch "$BackgroundMovie" | raw2image -ppm | ppmfilter --stack-image Frames/* period=1 --overlay position=0.0,0.0:100.0x100.0 | ppm2raw > "$OutputDir/Slideshow.dv"

    fi

  else

    cd "$WorkingDir/Frames"

    if [ -e "$WorkingDir/Soundtrack.wav" ]; then

      image2raw $WideScreenSwitch -A "$WorkingDir/Soundtrack.wav" * > "$OutputDir/Slideshow.dv"

    else

      image2raw $WideScreenSwitch * > "$OutputDir/Slideshow.dv"

    fi

  fi

  UpdateTask "Done."

  rm -f "$WorkingDir/Soundtrack.wav"
  DiskSpaceError "$WorkingDir" 
}


# One parameter: NumFrames.

ZoomIn ()
{
 echo "Executing Effect 'Zoom' for $1 Frames."
 
 # Calculate when we start fading out 
 if [ $1 -gt 25 ]; then
   FadeStart=`echo "($1 * 3) / 4" | bc`
   FadeFrames=$[$1 - $FadeStart]
 else
   FadeStart=$[$1 + 1]
 fi
 
 
 Zoom=0
 for ((Frame=1; Frame <= $1 ; Frame++))
    do
       UpdateTask "Creating Zoom Effect, frame $Frame of $1"

       # Zoom the slide and crop it.
       convert "$SlideA" -resize $[$Width + $Zoom + $Zoom]x$[$Height + $Zoom + $Zoom]! "$WorkingDir/zoomed.png"
       convert "$WorkingDir/zoomed.png" -crop ${Width}x${Height}+$Zoom+$Zoom "$WorkingDir/zoomed.png"

       # Apply shaky film effect
       if [ $FilmEffect == "true" ]; then
       XDisplacement=`echo "scale=4; ($RANDOM / 32767) * 0.7" | bc`
       YDisplacement=`echo "scale=4; ($RANDOM / 32767) * 0.7" | bc`
       convert "$WorkingDir/zoomed.png" -roll +$XDisplacement+$YDisplacement "$WorkingDir/zoomed.png"
       fi

       # Fade slide near the end
       if [ $Frame -gt $FadeStart ]; then
       Fade=`echo "100 - (($Frame - $FadeStart) * 100)/ $FadeFrames" | bc`
       eval "composite 100 -blend $Fade \"$WorkingDir/zoomed.png\" $Transparent -matte \"$WorkingDir/zoomed.png\""
       fi

       # Combine the slide with the background
       if [ $BackgroundType == "Movie" ]; then
         convert "$WorkingDir/zoomed.png" -resize 720x576! "$WorkingDir/Frames/Frame$NextFrameNumber.png"  
       else
	 convert \( "$WorkingDir/background.png" "$WorkingDir/zoomed.png" -compose over -composite \) -resize 720x576! -depth 8 +matte  "$WorkingDir/Frames/Frame$NextFrameNumber.png"
       fi

       Zoom=$[$Zoom + 1]
       GlobalFrameCounter=$[$GlobalFrameCounter + 1]
       NextFrameNumber=`echo $[1000000 + $GlobalFrameCounter] | tail -c 7`
       UpdateProgress `echo "($GlobalFrameCounter * 100)/$TotalFrames" | bc`

       echo -n "."
    done

  rm -f ""$WorkingDir/zoomed.png""
}

# One parameter: NumFrames.

Crossfade ()
{
 echo "Executing Effect 'Crossfade' for $1 Frames."
 
 Blend=0
 
 for ((Frame=1; Frame <= $1 ; Frame++))
    do
       UpdateTask "Creating Crossfade Effect, frame $Frame of $1"

       # Blend Slide A with Slide B
       composite -blend $Blend "$SlideA" "$SlideB" -matte "$OutputDir/blended.png"

       # Combine blend with background
       if [ $BackgroundType == "Movie" ]; then
         convert "$OutputDir/blended.png" -resize 720x576! "$WorkingDir/Frames/Frame$NextFrameNumber.png"  
       else
         convert \( "$WorkingDir/background.png" "$OutputDir/blended.png" -compose over -composite \) -resize 720x576! -depth 8 +matte "$WorkingDir/Frames/Frame$NextFrameNumber.png"
       fi

       Blend=`echo "scale=2; ($Frame * 100 )/ $1" | bc`
       GlobalFrameCounter=$[$GlobalFrameCounter + 1]
       NextFrameNumber=`echo $[1000000 + $GlobalFrameCounter] | tail -c 7`
       UpdateProgress `echo "($GlobalFrameCounter * 100)/$TotalFrames" | bc`

       echo -n "."
    done

  rm -f "$OutputDir/blended.png"
}


# One parameter: NumFrames.

FadeIn ()
{
 echo "Executing Effect 'FadeIn' for $1 Frames."
 
 Fade=0

 for ((Frame=1; Frame <= $1 ; Frame++))
    do
       UpdateTask "Creating Fade-in Effect, frame $Frame of $1"

       eval "composite -blend $Fade \"$SlideA\" $Transparent -matte \"$WorkingDir/faded.png\""

       # Apply shaky film effect
       if [ $FilmEffect == "true" ]; then
       XDisplacement=`echo "scale=4; ($RANDOM / 32767) * 0.7" | bc`
       YDisplacement=`echo "scale=4; ($RANDOM / 32767) * 0.7" | bc`
       convert "$WorkingDir/faded.png" -roll +$XDisplacement+$YDisplacement "$WorkingDir/faded.png"
       fi

       # Combine blend with background
       if [ $BackgroundType == "Movie" ]; then
         convert "$WorkingDir/faded.png" -resize 720x576! "$WorkingDir/Frames/Frame$NextFrameNumber.png"
       else
	 convert \( "$WorkingDir/background.png" "$WorkingDir/faded.png" -compose over -composite \) -resize 720x576! -depth 8 +matte "$WorkingDir/Frames/Frame$NextFrameNumber.png"
       fi

       Fade=`echo "scale=2; ($Frame * 100 )/ $1" | bc`
       GlobalFrameCounter=$[$GlobalFrameCounter + 1]
       NextFrameNumber=`echo $[1000000 + $GlobalFrameCounter] | tail -c 7`
       UpdateProgress `echo "($GlobalFrameCounter * 100)/$TotalFrames" | bc`

       echo -n "."
    done

  rm -f "$WorkingDir/faded.png"
}

# One parameter: Number of Frames.

Stable ()
{
 echo "Executing Effect 'None' for $1 Frames."
 
 if [ $BackgroundType == "Movie" ]; then
   convert "$SlideA" -resize 720x576! "$WorkingDir/Frames/Frame$NextFrameNumber.png"
 else
   composite "$SlideA" "$WorkingDir/background.png" -compose over "$WorkingDir/Frames/Frame$NextFrameNumber.png"
   convert "$WorkingDir/Frames/Frame$NextFrameNumber.png" -resize 720x576! -depth 8 +matte "$WorkingDir/Frames/Frame$NextFrameNumber.png"
 fi
  
 GlobalFrameCounter=$[$GlobalFrameCounter + 1]
 NextFrameNumber=`echo $[1000000 + $GlobalFrameCounter] | tail -c 7`
 LastFrame=`echo $[1000000 + $GlobalFrameCounter - 1] | tail -c 7`
 
 for ((Frame=1; Frame < $1 ; Frame++))
    do
       # Are we going to output identical frames or are they changing?
       if [ $FilmEffect == "true" ]; then

         UpdateTask "Creating Static Frames, frame $Frame of $1"

	 # Apply shaky film effect
	 XDisplacement=`echo "scale=4; ($RANDOM / 32767) * 0.6" | bc`
         YDisplacement=`echo "scale=4; ($RANDOM / 32767) * 0.6" | bc`
         convert "$SlideA" -roll +$XDisplacement+$YDisplacement  "$OutputDir/shaken.png"

         if [ $BackgroundType == "Movie" ]; then
	   convert "$OutputDir/shaken.png" -resize 720x576! "$WorkingDir/Frames/Frame$NextFrameNumber.png"
         else
	   convert "$WorkingDir/background.png" "$OutputDir/shaken.png" -compose over -resize 720x576! -depth 8 +matte  "$WorkingDir/Frames/Frame$NextFrameNumber.png"
	 fi

         GlobalFrameCounter=$[$GlobalFrameCounter + 1]
	 NextFrameNumber=`echo $[1000000 + $GlobalFrameCounter] | tail -c 7`
         UpdateProgress `echo "($GlobalFrameCounter * 100) / $TotalFrames" | bc`
	 
	 echo -n "."
	 
       else
         # All frames are identical. Just create some symbolic links.
	 ln -sf Frame$LastFrame.png "$WorkingDir/Frames/Frame$NextFrameNumber.png"
         GlobalFrameCounter=$[$GlobalFrameCounter + 1]
	 NextFrameNumber=`echo $[1000000 + $GlobalFrameCounter] | tail -c 7`
      fi
    done
 
 UpdateProgress `echo "($GlobalFrameCounter * 100)/$TotalFrames" | bc`
}

# One parameter: NumFrames.

FadeOut ()
{
 echo "Executing Effect 'FadeOut' for $1 Frames."
 
 Fade=100
 
 for ((Frame=1; Frame <= $1 ; Frame++))
    do
       UpdateTask "Creating Fade-out effect, frame $Frame of $1"

       # Blend slide with transparency
       eval "composite -blend $Fade \"$SlideA\" $Transparent -matte \"$WorkingDir/faded.png\""

       # Apply shaky film effect
       if [ $FilmEffect == "true" ]; then
       XDisplacement=`echo "scale=4; ($RANDOM / 32767) * 0.7" | bc`
       YDisplacement=`echo "scale=4; ($RANDOM / 32767) * 0.7" | bc`
       convert "$WorkingDir/faded.png" -roll +$XDisplacement+$YDisplacement "$WorkingDir/faded.png"
       fi

       # Combine slide with background
       if [ $BackgroundType == "Movie" ]; then
         convert "$WorkingDir/faded.png" -resize 720x576! "$WorkingDir/Frames/Frame$NextFrameNumber.png"
       else
         composite  "$WorkingDir/Frames/Frame$NextFrameNumber.png"
	 convert \( "$WorkingDir/background.png" "$WorkingDir/faded.png" -compose over -composite \) -resize 720x576! -depth 8 +matte  "$WorkingDir/Frames/Frame$NextFrameNumber.png"
       fi

       Fade=`echo "scale=2; 100 - ($Frame * 100 )/ $1" | bc`
       GlobalFrameCounter=$[$GlobalFrameCounter + 1]
       NextFrameNumber=`echo $[1000000 + $GlobalFrameCounter] | tail -c 7`
       UpdateProgress `echo "($GlobalFrameCounter * 100)/$TotalFrames" | bc`

       echo -n "."
    done
  
  rm -f "$WorkingDir/faded.png"
}

# This function creates transparent dummy frames that are needed
# when a background movie is used that is longer than the slideslow.

CreateBlankFrames ()

{
  if [ $BackgroundType == "Movie" ]; then

    NumBackgroundFrames=$[ `du -b "$BackgroundMovie" | awk '{print $1}'` / $DVFrameSize ]

    if [ $NumBackgroundFrames -gt $TotalFrames ]; then 

      NumBlankFrames=$[$NumBackgroundFrames - $TotalFrames]

      echo "Generating $NumBlankFrames blank frames."

      convert -size 720x576 xc:'#000000FF' "$WorkingDir/Frames/Frame$NextFrameNumber.png"
      LastFrame=$NextFrameNumber
      GlobalFrameCounter=$[$GlobalFrameCounter + 1]
      NextFrameNumber=`echo $[1000000 + $GlobalFrameCounter] | tail -c 7`

      for ((Frame=1; Frame <= $NumBlankFrames ; Frame++))
        do

	 ln -sf Frame$LastFrame.png "$WorkingDir/Frames/Frame$NextFrameNumber.png"
         GlobalFrameCounter=$[$GlobalFrameCounter + 1]
	 NextFrameNumber=`echo $[1000000 + $GlobalFrameCounter] | tail -c 7`

	done

    fi

  fi
}


# PROGRAM STARTS HERE

RunChecks

if [ -z "$1" ]; then
 echo "You did not provide a configuration file."
 echo
 echo "USAGE: SlideShow ConfigFile.conf"
 exit -1
fi

DialogDCOPProgressCommand="$2"
DialogDCOPTaskCommand="$3"

# Read configuration file

NumSlides=$(cat "$1" | grep -E ^NumSlides | cut -d = -f 2)

for ((Clip=1; Clip <= NumSlides ; Clip++))
 do
 SlidePath[$Clip]="$( cat "$1" | grep -F SlidePath[$Clip] | cut -d \" -f 2)"
 echo "Path: ${SlidePath[$Clip]}"
 SlideLength[$Clip]="$( cat "$1" | grep -F SlideLength[$Clip] | cut -d \" -f 2)"
 echo "Length: ${SlideLength[$Clip]}"
 SlideFlickerReduction[$Clip]="$( cat "$1" | grep -F SlideFlickerReduction[$Clip] | cut -d \" -f 2)"
 echo "Flickerreduction: ${SlideFlickerReduction[$Clip]}"
 SlideEffect[$Clip]="$( cat "$1" | grep -F SlideEffect[$Clip] | cut -d \" -f 2)"
 echo "Effect: ${SlideEffect[$Clip]}"
 done

OutputDir="$( cat "$1" | grep -E ^OutputDir= | cut -d \" -f 2)"
WideScreen="$( cat "$1" | grep -E ^Widescreen= | cut -d \" -f 2)"
#FilmEffect="$( cat "$1" | grep -F FilmEffect | cut -d \" -f 2)"
FilmEffect="false"
BackgroundType="$( cat "$1" | grep -E ^BackgroundType | cut -d \" -f 2)"
BackgroundImage="$( cat "$1" | grep -E ^BackgroundImage | cut -d \" -f 2)"
BackgroundMovie="$( cat "$1" | grep -E ^BackgroundMovie | cut -d \" -f 2)"
SoundType="$( cat "$1" | grep -E ^SoundType | cut -d \" -f 2)"
SoundtrackFile="$( cat "$1" | grep -E ^SoundtrackFile | cut -d \" -f 2)"

mkdir -p "$OutputDir"
WorkingDir="$OutputDir/DVSlideshow_Temporary_Files_$RANDOM"
DiskSpaceWarning "$OutputDir" $NumSlides

if [ $WideScreen == "true" ]; then
Width=1024
else
Width=768
fi
Height=576

if [ "$BackgroundType" == "none" ]; then
  Transparent="-size ${Width}x${Height} xc:black"
else
  Transparent="-size ${Width}x${Height} xc:'#000000FF'"
fi

# Create working directories

rm -f -R "$WorkingDir"
mkdir -p "$WorkingDir"
mkdir -p "$WorkingDir/Frames"

# Create background image

if [ $BackgroundType == "None" ]; then
  if [ $WideScreen == "true" ]; then
    convert -size ${Width}x${Height} xc:black "$WorkingDir/background.png"
  else
    convert -size 768x576 xc:black "$WorkingDir/background.png"
  fi
elif [ $BackgroundType == "Movie" ]; then
  if [ $WideScreen == "true" ]; then
    convert -size ${Width}x${Height} xc:'#000000FF' "$WorkingDir/background.png"
  else
    convert -size 768x576 xc:'#000000FF' "$WorkingDir/background.png"
  fi
fi

# Calculate total output frames 
TotalFrames=0;
for ((Slide=1; Slide <= $NumSlides ; Slide++))
    do
    TotalFrames=`echo "$TotalFrames + (${SlideLength[$Slide]} * 25 ) / 1000" | bc`
    done
echo "TotalFrames: $TotalFrames"

ProcessSlides

UpdateTask "Generating Frames..."
UpdateProgress 0


# Global Frame Counter. Should always start at 0.
GlobalFrameCounter=0
NextFrameNumber=`echo $[1000000 + $GlobalFrameCounter] | tail -c 7`

DiskSpaceWarning "$OutputDir" $[$TotalFrames / 2]


# Main program loop. Process all slides.

for ((Slide=1; Slide <= NumSlides ; Slide++))
   do
   SlideA="$WorkingDir/Slides/Slide$Slide".png
   SlideB="$WorkingDir/Slides/Slide`echo $[$Slide - 1]`.png"
   if [ $Slide == 0 ]; then
   SlideB=$SlideA
   fi

   Length=`echo "( ${SlideLength[$Slide]} * 25 ) / 1000" | bc`
   
   if [ "${SlideEffect[$Slide]}" == "None" ]; then
     Stable $Length
   fi
   if [ "${SlideEffect[$Slide]}" == "Fade-In" ]; then
     FadeIn $Length
   fi
   if [ "${SlideEffect[$Slide]}" == "Fade-Out" ]; then
     FadeOut $Length
   fi
   if [ "${SlideEffect[$Slide]}" == "Cross-Fade" ]; then
     Crossfade $Length
   fi
   if [ "${SlideEffect[$Slide]}" == "Zoom" ]; then
     ZoomIn $Length
   fi
   DiskSpaceError "$OutputDir"
   done

CreateBlankFrames
CreateAudioTrack
EncodeToDV

rm -R -f "$WorkingDir"

UpdateTask "Done."
UpdateProgress 0

echo "Done. Have a nice day!"